# Abstract Syntax Tree
An Abstract Syntax Tree (AST) is a [tree](tree.md) representation of a
program, directly representing its nested syntactical structure. It is called
abstract because it only represents the exact structure of the syntax, unlike a
concrete syntax tree.

An AST is the first [intermediate representation](intermediate_representation.md) used
by a programming language, being created by a [parser](parser.md). Usually this tree is
later decorated with additional information by the compiler, such as typing information in latter compilation
phases, in this case the semantic analysis phase.

[Lisp](lisp.md) famously uses a very simple syntax for its source code, [S-Expressions](sexpression.md), which makes
the AST evident and makes Lisp [homoiconic](homoiconic.md).

For this C function that calculates the factorial of *n*:
```c
long long
fact(int n)
{
    long long res = 1;
    while (n > 0) {
        res *= n;
        --n;
    }
    return res;
}
```
the AST would be similar to this:
```
                                                  [ function_decl      ]                        
                                                  [ name: fact         ]                      
                                                  [ rettype: long long ]
                                                             |
                                                     .-------+------------------. 
                                                     |                    [ arg_list ]
                                                     |                          |
                                                     |                       [ var ]
                                                     |                        /   \
                                                     |                       /     \
                                                     |            [ ident: n ]     [ type: int ]
                                                     | 
                                                     |
                                                statements
                                                     |
         .-------------------------------------------+---------------------------------------------.
         |                                           |                                             |   
  [ declaration  ]                            [ while_stmt ]                               [ return_stmt ]   
  [ ident: res   ]                                   |                                             |
          |                        .-----------------+-----------------.                    [ ident: res ]
      assgn_expr            condition_expr                        statements                                  
          |                        |                       .___________|___________.
        [ 1 ]                    [ > ]                     |                       |
                                  / \             [ assignment ]              [ decrement ]
                                 /   \            [ ident: res ]              [ ident: n  ]
                      [ ident: n ]   [ 0 ]               |
                                                       expr
                                                         |
                                                       [ * ]
                                                        / \
                                                       /   \
                                          [ ident: res ]   [ ident: n ]

```
